package SpaceRP_Building
{
	function fxDtsBrick::delete(%brick)
	{
		if(isObject(%brick.light)) %brick.light.delete();
		if(isObject(%brick.emitter)) %brick.emitter.delete();
		if(isObject(%brick.item)) %brick.item.delete();
		if(%brick.isPlanted() && %brick.getDatablock().getName() $= "brickStationCoreData")
		{
			%group = %brick.brickgroup;
			%count = ClientGroup.getCount();
			for(%i=0;%i<%count;%i++)
			{
				%this = ClientGroup.getObject(%i);
				if(%this.hasRSClient) commandToClient(%this, 'RSC_UpdateDirMarker', "Core_"@%brick.getID(), "DELETE");
			}
			Parent::delete(%brick);
			if(!isObject(%group)) return;
			for(%i=%group.getCount()-1;%i>=0;%i--)
				%group.getObject(%i).delete();
			%group.delete();
		}
		else Parent::delete(%brick);
	}
	
	function fxDtsBrick::killBrick(%brick)
	{
		if(isObject(%group = %brick.asteroidGroup))
			for(%i=%group.getCount()-1;%i>=0;%i--)
				%group.getObject(%i).delete();
		else
		{
			if(isObject(%brick.light)) %brick.light.delete();
			if(isObject(%brick.emitter)) %brick.emitter.delete();
			if(isObject(%brick.item)) %brick.item.delete();
			serverPlay3D(BrickBreakSound, %brick.getPosition());
			%brick.delete();
		}
	}
	
	function fxDtsBrick::onRemove(%brick)
	{
		if(%brick.isSteel)
		{
			if(isObject(%brick.light)) %brick.light.delete();
			if(isObject(%brick.emitter)) %brick.emitter.delete();
			if(isObject(%brick.item)) %brick.item.delete();
			if(isObject(%cl = %brick.getGroup().client))
			{
				%maxVal = (%cl.pack == 4 ? 150 : 25) - %cl.totalMats;
				%steel = %brick.getVolume() / 100;
				if(%steel <= %maxVal) { %cl.matSteel += %steel; %cl.totalMats += %steel; }
				else
				{
					%cl.matSteel += %maxVal; %cl.totalMats += %maxVal;
					%brick.stationCore.matSteel += %steel - %maxVal;
				}
				commandToClient(%cl, 'UpdateMaterial', "Steel", %cl.matSteel);
			}
			else %brick.stationCore.matSteel += %brick.getVolume() / 100;
		}
		Parent::onRemove(%brick);
	}
	
	function HammerImage::onFire(%image, %obj, %slot)
	{
		%eye = %obj.getEyePoint();
		%dir = %obj.getEyeVector();
		%mask = $Typemasks::fxBrickAlwaysObjectType | $Typemasks::PlayerObjectType
			| $Typemasks::TerrainObjectType | $Typemasks::StaticObjectType
			| $Typemasks::VehicleObjectType;
		%ray = containerRaycast(%eye, vectorAdd(%eye, vectorScale(%dir, 8)), %mask, %obj);
		if(isObject(%hit = firstWord(%ray)))
		{
			serverPlay3DSpace(HammerHitSound, getWords(%ray, 1, 3));
			new Projectile()
			{
				datablock = HammerProjectile;
				initialPosition = vectorAdd(getWords(%ray, 1, 3), vectorScale(getWords(%ray, 4, 6), 0.05));
				initialVelocity = vectorScale(getWords(%ray, 4, 6), -100);
			};
			if(%hit.getType() & $Typemasks::fxBrickAlwaysObjectType)
			{
				if(%hit.getDatablock().getName() $= "brickStationCoreData") return;
				if(getTrustLevel(%obj, %hit) >= $TrustLevel::Hammer) %hit.killBrick();
			}
		}
	}
	
	function servercmdPlantBrick(%cl)
	{
		if(!isObject(%cl) || !isObject(%pl = %cl.player) || !isObject(%temp = %pl.tempbrick))
			return -1;
		if(!$RSM::Sandbox && %cl.matSteel < %temp.getVolume() / 100)
		{ %cl.centerPrint("\c0You don't have enough Steel to build that brick!", 3); return -1; }
		if((%brickData = %temp.getDatablock()).getName() !$= "brickStationCoreData")
		{
			if(!isObject(StationCores)) { %cl.centerPrint("\c0Can't build there; start with a Station Core!", 3); return -1; }
			%pos = %cl.player.tempbrick.getPosition();
			%count = StationCores.getCount();
			for(%i=0;%i<%count;%i++)
			{
				%core = StationCores.getObject(%i);
				if(vectorDist(%pos, %core.getPosition()) >= 100) continue;
				%finalCore = %core; break;
			}
			if(!isObject(%finalCore)) { %cl.centerPrint("\c0Can't build there; start with a Station Core!", 3); return -1; }
			if(%finalCore.stationName $= "Unnamed Station") { %cl.centerPrint("\c0Can't build there; that station isn't named yet!", 3); return -1; }
			if(getTrustLevel(%cl, %group = %finalCore.getGroup()) < $TrustLevel::Full)
			{ %cl.centerPrint("\c0Can't build there; "@(%group.name!$=""?%group.name:"the owner of "@%finalCore.stationName)@" doesn't trust you enough!", 3); return -1; }
			//Ultimately, this brick is still too far from the station, and must be considered out of bounds.
			if(%temp.getMaxDist(%finalCore.getPosition()) > 100) { %cl.centerPrint("\c0Can't build there; start with a Station Core!", 3); return -1; }
		}
		else if(vectorLen(%pl.getVelocity()) > 0.75)
		{ %cl.centerPrint("\c0You must be holding stationary to start a Station!", 3); return -1; }
		%data = createBrick(isObject(%finalCore) ? %coreGroup = %finalCore.getGroup() : %cl, %brickData, %temp.getPosition(),
			%temp.getColorID(), %temp.getAngleID());
		%brick = getField(%data, 0); %error = getField(%data, 1);
		if(%error != 2)						//Can't seem to get the errors to work right.
		{							//We already know planting this brick results in an error,
			%brick.delete();				//so just call the parent, let it try (causing the error),
			%brick = Parent::servercmdPlantBrick(%cl);	//then return the brick that it spawns. In the case of there
			if(isObject(%brick) && isObject(%finalCore))	//not being an error, the brick wasn't floating - don't overwrite.
			{
				%brick.isSteel = 1;
				%coreGroup.add(%brick);
				%brick.stationCore = %finalCore;
				%finalCore.brickgroup.add(%brick);
				%volume = %brick.getVolume() / 100;
				%cl.matSteel -= %volume; %cl.totalMats -= %volume;
				commandToClient(%cl, 'UpdateMaterial', "Steel", %cl.matSteel);
				//serverPlay3DSpace(SpaceBrickPlantSound, %brick.getPosition());
				return %brick;
			}
			else if(!isObject(%finalCore)) { %cl.centerPrint("\c0Can't build there; start with a Station Core!", 3); return -1; }
		}
		if(isObject(%brick) && isObject(%finalCore))
		{
			%brick.isSteel = 1;
			%brick.stationCore = %finalCore;
			%finalCore.brickgroup.add(%brick);
			%volume = %brick.getVolume() / 100;
			if(!$RSM::Sandbox)
			{ %cl.matSteel -= %volume; %cl.totalMats -= %volume; }
			if(%brick.getDatablock().category $= "Baseplates")
				%brick.isBaseplate = 1;
			%cl.undoStack.push(%brick TAB "PLANT");
			if(%brickData.hasPrint && %cl.lastPrint[%brickData.printAspectRatio] !$= "")
				%brick.setPrint(%cl.lastPrint[%brickData.printAspectRatio]);
			//serverPlay3DSpace(SpaceBrickPlantSound, %brick.getPosition());
			return %brick;
		}
		else if(!isObject(%finalCore)) { %cl.centerPrint("\c0Can't build there; start with a Station Core!", 3); return -1; }
	}
	
	function servercmdUndoBrick(%cl)
	{
		%pos = (%stack = %cl.undoStack).head - 1; if(%pos == -1) %pos = 511;
		if(getField(%val = %stack.val[%pos], 1) !$= "PLANT") { Parent::servercmdUndoBrick(%cl); return; }
		else { serverPlay3D(BrickBreakSound, (%brick = getField(%val, 0)).getPosition()); %brick.delete(); }
		%stack.head = %pos;
	}
};
activatePackage("SpaceRP_Building");

function fxDtsBrick::getVolume(%brick)
{ %data = %brick.getDatablock(); return %data.brickSizeX * %data.brickSizeY * %data.brickSizeZ; }

function brickStationCoreData::onAdd(%data, %brick)
{
	if(!%brick.isPlanted()) return;
	if(!isObject(StationCores)) new SimSet(StationCores);
	%count = StationCores.getCount();
	%pos = %brick.getPosition();
	for(%i=0;%i<%count;%i++)
	{
		if(vectorDist(%pos, (%core = StationCores.getObject(%i)).getPosition()) < $RSM::AsteroidDist * 5)
		{ %conflictName = %core.stationName; %conflictCount++; }
	}
	if(%conflictCount >= 1) { %brick.schedule(0, "destroyStationCore", %conflictCount == 1 ? %conflictName : %conflictCount@" other stations"); return; }
	%count = AsteroidGroup.getCount(); for(%i=0;%i<%count;%i++) if(vectorDist(%pos, AsteroidGroup.getObject(%i).centerPos) < $RSM::AsteroidDist * 2) %asteroidCount++;
	if(%asteroidCount >= 1) { %brick.schedule(0, "destroyStationCore", %asteroidCount@" asteroid"@(%asteroidCount==1?"":"s")); return; }
	%brick.schedule(0, "nameStationCore");
}

function fxDtsBrick::destroyStationCore(%brick, %reason)
{ %brick.schedule(0, "delete"); %brick.getGroup().client.centerPrint("Can't build a Core there; too close to "@%reason@"!", 3); }

function fxDtsBrick::nameStationCore(%brick)
{
	if(%brick.isLoadedCore) return;
	%cl = %brick.getGroup().client; if(isObject(%cl.pendingCore)) { %brick.schedule(0, "delete"); return; }
	%cl.chatMessage("\c6Planted a Station Core! Please use the \"/nameCore Station Name\" command to name the station!");
	StationCores.add(%cl.pendingCore = %brick); %brick.stationName = "Unnamed Station";
	%brick.destroyAFKCore = %brick.schedule(90000, "delete");
}

function servercmdNameCore(%cl, %n0, %n1, %n2, %n3, %n4, %n5, %n6, %n7, %n8, %n9)
{
	if(!isObject(%brick = %cl.pendingCore)) { %cl.chatMessage("\c6You don't have a pending Core!"); return; }
	cancel(%brick.destroyAFKCore); %brick.destroyAFKCore = %brick.schedule(30000, "delete");
	for(%i=0;%i<10;%i++) %name = %name SPC %n[%i]; %name = trim(%name);
	if(%name $= "Unnamed Station") { %cl.centerPrint("\c0Can't name your station that!", 3); } %brick.pendingName = %name;
	commandToClient(%cl, 'messageBoxYesNo', "Station Name Confirmation", "Are you sure you want to name your station<br>\""@%name@"\"?<br><br>This cannot be changed later!", 'ConfirmCoreName');
}

function servercmdConfirmCoreName(%cl)
{
	if(!isObject(%brick = %cl.pendingCore)) { %cl.chatMessage("\c6You don't have a pending Core!"); return; }
	cancel(%brick.destroyAFKCore); %brick.stationName = %brick.pendingName; %brick.pendingName = ""; %cl.pendingCore = "";
	%brick.brickgroup = new SimSet();
}

function createBrick(%cl, %data, %pos, %color, %angleID)
{
	if(!isObject(%data) || %data.getClassName() !$= "fxDTSBrickData")
		return -1;
	if(getWordCount(%pos) != 3)
		return -1;
	if(%angleID $= "")
		%angleID = 0;
	if(isObject(%cl) && (%cl.getClassName() $= "GameConnection" | %cl.getClassName() $= "AIConnection"))
	{
		%blid = %cl.bl_id;
		if(%blid $= "")
			%blid = -1;
		%flag = 1;
	}
	else if(isObject(%cl) && %cl.getClassName() $= "SimGroup" && MainBrickgroup.isMember(%cl))
	{
		%group = %cl;
		%cl = %group.client;
		%blid = %group.bl_id;
	}
	else %cl = 0;
	switch(%angleID)
	{
		case 0:
			%rot = "1 0 0 0";
		case 1:
			%rot = "0 0 1 90";
		case 2:
			%rot = "0 0 1 180";
		case 3:
			%rot = "0 0 -1 90";
	}
	(%brick = new fxDtsBrick()
	{
		client = %cl;
		colorFxID = 0;
		colorID = %color;
		datablock = %data;
		isPlanted = 1;
		position = getWord(%pos, 0) SPC getWord(%pos, 1) SPC getWord(%pos, 2);
		rotation = %rot;
		shapeFxID = 0;
		stackBL_ID = %blid;
	}).angleID = %angleID;
	%err = %brick.plant();
	%brick.setTrusted(1);
	missionCleanup.add(%brick);
	if(%flag) %cl.brickgroup.add(%brick);
	else if(isObject(%group)) %group.add(%brick);
	return %brick TAB %err;
}