registerOutputEvent("fxDtsBrick", "setVentRate", "list Oxygen 0 CarbonDioxide 1 Nitrogen 2 Ammonia 3 Acetone 4\tfloat -0.2 3 0.1 0");
function fxDtsBrick::setVentRate(%brick, %gas, %rate)
{
	if(%brick.getDatablock().getName() !$= "brickAirVentData") return;
	%brick.gasVentRate[%gas] = %rate;
}

function fxDtsBrick::VentTick(%brick)
{
	return;   //Gasseous simulation doesn't yet work correctly :(
	cancel(%brick.VentTick);
	%now = getSimTime();
	//The vents need to use station mats to create the gas
	for(%i=0;%i<5;%i++)
	{
		if(%brick.gasVentRate[%i] > 0)
		%emit = vectorDist(%brick.lastAction[%i], %now) / 1000;
		if((%brick.gasVentRate[%i] * %emit) >= 1)
		{
			%brick.lastAction[%i] = %now;
			if(mAbs(%i - 1) != 1) continue;
			if(%brick.stationCore.mat[%i?"Nitrogen":"Oxygen"] < 0.01) continue;
			%angle = getWord(%brick.getTransform(), 6);
			%nozzle = vectorAdd(%brick.getPosition(), mCos(%angle) SPC mSin(%angle)@" 0");
			%jetDir = vectorNormalize(vectorSub(%nozzle, %brick.getPosition()));
			if(%brick.stationCore.getGasPressure(%nozzle, %i) < 5)
			{
				%brick.stationCore.mat[%i?"Nitrogen":"Oxygen"] -= 0.01;
				CreateGasParticle(%brick.stationCore, %nozzle, vectorAdd(vectorScale(%jetDir, 20), vectorScale(getUnitVector(), 2.5)), %i);
			}
		}
	}
	%brick.VentTick = %brick.schedule(200, "VentTick");
}

function fxDtsBrick::getGasPressure(%core, %pos, %gas)
{
	//Gasseous simulation doesn't yet work correctly :(
	//This is a really simple check to make sure the player is "inside" the "station".
	if(vectorDist(%core.getPosition(), %pos) >= 100) return 0;
	%mask = $Typemasks::fxBrickAlwaysObjectType;
	%edge = vectorAdd(vectorScale(vectorNormalize(vectorSub(%pos, %center = %core.getPosition())), 100), %center);
	%ray = containerRaycast(%edge, %pos, %mask);
	while(isObject(%hit = firstWord(%ray)) && !%hit.isColliding() && %abort < 50)
	{ %ray = containerRaycast(vectorAdd(getWords(%ray, 1, 3), vectorScale(getWords(%ray, 4, 6), -0.05)), %pos, %mask); %abort++; }
	//talk(%abort SPC %hit SPC %core);
	if(%abort == 50) return 0;
	else if(isObject(%hit) && %hit != %core) return 10;
	else return 0;
	if(!isObject(%gasses = %core.gasGroup)) return 0;
	%count = %gasses.getCount();
	%mask = $Typemasks::fxBrickObjectType;
	for(%i=0;%i<%count;%i++)
	{
		%part = %gasses.getObject(%i);
		if(!isObject(firstWord(containerRaycast(%pos, %part.position, %mask))))
		{
			%hotspot = vectorAdd(%part.position, %part.velocity);
			%dist = vectorDist(%pos, %hotspot); if(%dist > 30) continue;
			%pressure += mCos(%dist / 60 * $PI);
		}
	}
	return %pressure + 0;
}

function CreateGasParticle(%core, %pos, %vel, %type)
{
	%gas = new ScriptObject()
	{
		class = "GasParticleSO";
		gasType = %type;
		position = %pos;
		velocity = %vel;
	};
	if(!isObject(%core.gasGroup))
		%core.gasGroup = new SimSet();
	%core.gasGroup.add(%gas);
	%gas.GasTick = %gas.schedule(200, "Tick", %core, getSimTime(), getSimTime());
	return %gas;
}

function GasParticleSO::Tick(%gas, %core, %start, %last)
{
	cancel(%gas.GasTick);
	%now = getSimTime();
	if(vectorDist(%start, %now) > 45000) { %gas.delete(); %core.matOxygen = getMin(%core.matOxygen + 0.01, 10000); return; }
	%time = vectorDist(%last, %now) / 1000;
	//Add player hitting for dangerous gasses
	if(%gas.Move(%gas.position, vectorNormalize(%gas.velocity), %time * vectorLen(%gas.velocity)) == -1) return;
	if(vectorDist(%gas.position, %core.getPosition()) > 50) { %gas.delete(); return; }
	%gas.GasTick = %gas.schedule(200, "Tick", %core, %start, %now);
}

function GasParticleSO::Move(%gas, %start, %dir, %dist, %damage, %ignore, %depth)
{
	if(%depth >= 20) return 0;
	%mask = $Typemasks::fxBrickAlwaysObjectType;// | $Typemasks::PlayerObjectType;
	%hit = firstWord(%ray = containerRaycast(%start, vectorAdd(%start, vectorScale(%dir, %dist)), %mask, %ignore));
	if(isObject(%hit) && (%hit.getType() & $Typemasks::fxBrickAlwaysObjectType) && !%hit.isColliding())
	{
		%newStart = vectorAdd(getWords(%ray, 1, 3), vectorScale(%dir, 0.01));
		return %gas.Move(%newStart, %dir, %dist - vectorDist(%start, %newStart), %damage, %hit, %depth + 1);
	}
	else if(!isObject(%hit)) { %gas.position = vectorAdd(%start, vectorScale(%dir, %dist)); return 0; }
	else
	{
		if(%hit.getDatablock().getName() $= "brickAirVentData")
			if(%hit.gasVentRate[%gas.gasType] < 0)
			{
				%absorb = vectorDist(%hit.lastAction[%gas.gasType], %now = getSimTime()) / 1000;
				if((%hit.gasVentRate[%gas.gasType] * %absorb) <= -1)
				{ %hit.lastAction[%gas.gasType] = %now; %gas.delete(); return -1; }
			}
		%hitPos = getWords(%ray, 1, 3);
		%normal = getWords(%ray, 4, 6);
		%normE = mAbs(getWord(%normal, 0));
		%normN = mAbs(getWord(%normal, 1));
		%normU = mAbs(getWord(%normal, 2));
		if(%normE <= getMin(%normN, %normU))
			%axis0 = "1 0 0";
		else if(%normN <= %normU)
			%axis0 = "0 1 0";
		else	%axis0 = "0 0 1";
		%axis1 = vectorCross(%normal, %axis0);
		%relDir = Solve3DMatrix(%dir, %normal, %axis0, %axis1);
		%newDir = vectorAdd(vectorScale(%normal, -getWord(%relDir, 0)),
			vectorAdd(vectorScale(%axis0, getWord(%relDir, 1)),
			vectorScale(%axis1, getWord(%relDir, 2))));
		%gas.velocity = vectorScale(%newDir, vectorLen(%gas.velocity));
		return %gas.Move(%hitPos, %newDir, %dist - vectorDist(%start, %hitPos), %damage, %hit, %depth + 1) SPC %hit;
	}
}

datablock fxDtsBrickData(brickAirVentData)
{
	brickFile = "./Bricks/Air Vent.blb";
	category = "Space Mod";
	subcategory = "Station Parts";
	uiName = "Air Vent";
};

function brickAirVentData::onAdd(%data, %brick)
{
	//Pressure needs to equalize - the brick can only "push" in so much
	//Oxygen poisoning starts at roughly 500 millibars
	%brick.gasVentRate0 = 3;   //0.8; (Use 0.8 once nitrogen is implemented)
	%brick.gasVentRate1 = -0.2;
	//%brick.gasVentRate2 = 3;
	%brick.gasVentRate3 = -0.2;
	%brick.gasVentRate4 = -0.2;
	%brick.VentTick();
}

datablock AudioProfile(OxyHiss1)
{
	filename = "./Oxygen Hiss/Oxygen Hiss 1.wav";
	description = AudioClosestLooping3D;
	preload = false;
};

datablock AudioProfile(OxyHiss2 : OxyHiss1)
{ filename = "./Oxygen Hiss/Oxygen Hiss 2.wav"; };

datablock AudioProfile(OxyHiss3 : OxyHiss1)
{ filename = "./Oxygen Hiss/Oxygen Hiss 3.wav"; };

datablock AudioProfile(OxyHiss4 : OxyHiss1)
{ filename = "./Oxygen Hiss/Oxygen Hiss 4.wav"; };

datablock AudioProfile(OxyHiss5 : OxyHiss1)
{ filename = "./Oxygen Hiss/Oxygen Hiss 5.wav"; };