package SpaceMod_SaveOnExit
{
	function onExit()
	{
		RSM_SaveToFile("config/server/RSM Save.rss");
		Parent::onExit();
	}
};
activatePackage("SpaceMod_SaveOnExit");

function RSM_LoadFromFile(%path)
{
	//Make everything here be color-correct (IE, color ID 0 isn't just assumed to be color ID 0)
	if(!isFile(%path)) return;
	if(!isObject(%main = MainBrickgroup))
		schedule(16, 0, "RSM_LoadFromFile", %path);
	%file = new FileObject(); %file.openForRead(%path);
	while(!%file.isEOF())
	{
		%line = %file.readLine();
		if(firstWord(%line) $= "Station") { %stationName = restWords(%line); continue; }
		%line = stripLeadingTags(%line);
		if(strLen(%line) >= 2 && getSubStr(%line, 0, 2) $= "+-")
		{
			%val = getWord(%line, 0); %val = getSubStr(%val, 2, strLen(%val) - 2);
			if(!isObject(%brick) && %val !$= "CORE") continue;
			switch$(%val)
			{
				case "CORE":
					%blid = getWord(%line, 1);
					if(!isObject(StationCores)) new SimSet(StationCores);
					if(isObject("Brickgroup_"@%blid)) %group = ("Brickgroup_"@%blid).getID();
					else
					{
						if(%nameOf[%blid] !$= "") %name = %nameOf[%blid];
						MainBrickgroup.add(%group = new SimGroup() { bl_id = %blid; name = %name; });
						%group.setName("Brickgroup_"@%blid);
					}
					%core = createBrick(%group, brickStationCoreData, vectorScale(getWords(%line, 2, 4), $RSM::AsteroidDist / 60), getWord(%line, 5));
					%center = %core.getPosition();
					if((%err = getField(%core, 1)) && %err != 2) { %core.delete(); %brick = ""; continue; } %core.isSteel = 1;
					%core.setColorFX(getWord(%line, 6)); %core.setShapeFX(getWord(%line, 7));
					%core.setRaycasting(getword(%line, 8)); %core.setColliding(getWord(%line, 9)); %core.setRendering(getWord(%line, 10));
					StationCores.add(%core);
					%core.isLoadedCore = 1;
					%core.stationName = %stationName;
					%core.brickgroup = new SimSet();
					%brick = %core;
				case "MAT":
					%mat = restWords(getField(%line, 0));
					%amt = getField(%line, 1);
					%core.mat[%mat] = mFloatLength(%amt, 3);
				case "NTOBJECTNAME": if(isObject(%brick)) %brick.setNTObjectName(restWords(%line));
				case "EMITTER":
					%uiName = restWords(getSubStr(%line, 0, %pos = strPos(%line, "\"")));
					%brick.setEmitter($uiNameTable_Emitters[%uiName]);
					%brick.setEmitterDirection(getSubStr(%line, %pos + 2, strLen(%line) - (%pos + 2)));
				case "LIGHT":
					%uiName = restWords(getSubStr(%line, 0, %pos = strPos(%line, "\"")));
					%brick.setLight($uiNameTable_Lights[%uiName]);
				case "ITEM":
					%uiName = restWords(getSubStr(%line, 0, %pos = strPos(%line, "\"")));
					%line = getSubStr(%line, %pos + 2, strLen(%line) - (%pos + 2));
					%brick.setItem($uiNameTable_Items[%uiName]);
					%brick.setItemPosition(getWord(%line, 0));
					%brick.setItemDirection(getWord(%line, 1));
					%brick.setItemRespawnTime(getWord(%line, 2));
				case "AUDIOEMITTER":
					%uiName = restWords(getSubStr(%line, 0, %pos = strPos(%line, "\"")));
					%brick.setMusic($uiNameTable_Music[%uiName]);
				case "VEHICLE":
					%uiName = restWords(getSubStr(%line, 0, %pos = strPos(%line, "\"")));
					%brick.setVehicle($uiNameTable_Vehicles[%uiName]);
					%brick.setRecolorVehicle(getSubStr(%line, %pos + 2, strLen(%line) - (%pos + 2)));
				case "EVENT":
					%ind = -1+%brick.numEvents++;
					%data = getFields(%line, 1, getFieldCount(%line) - 1);
					%brick.eventEnabled[%ind] = getField(%data, 0);
					%brick.eventDelay[%ind] = getField(%data, 1);
					%brick.eventInput[%ind] = getField(%data, 2);
					%inputIDX = InputEvent_GetInputEventIDX(getField(%data, 2));
					%brick.eventInputIDX[%ind] = %inputIDX;
					%target = getField(%data, 3);
					%brick.eventTarget[%ind] = %target;
					%targetIDX = -1; %type = "fxDtsBrick";
					%targets = $InputEvent_TargetListfxDTSBrick_[%inputIDX];
					for(%i=0;%i<getFieldCount(%targets);%i++)
						if(getWord(getField(%targets, %i), 0) $= %target)
						{ %targetIDX = %i; %type = getWord(getField(%targets, %i), 1); break; }
					%brick.eventTargetIDX[%ind] = %targetIDX; %offset = 0;
					if(%targetIDX == -1) { %brick.eventNT[%ind] = getField(%data, 4); %offset = 1; }
					%brick.eventOutput[%ind] = getField(%data, 4 + %offset);
					%outputIDX = OutputEvent_GetOutputEventIDX(%type, getField(%data, 4 + %offset));
					%brick.eventOutputIDX[%ind] = %outputIDX;
					for(%i=0;%i<5;%i++) %brick.eventOutputParameter[%ind, %i + 1] = getField(%data, %i + %offset + 5);
					%brick.eventAppendClient[%ind] = $OutputEvent_AppendClient[%type, %outputIDX];
				case "VALUE":
					%data = restWords(%line);
					%len = strLen(%data);
					%readVar = 0;
					%readValue = 0;
					%varName = "";
					%value = "";
					for(%i=0;%i<%len;%i++)
					{
						%char = getSubStr(%data, %i, 1);
						if(%escaped)
						{
							if(%readVar == 1) %varName = %varName @ %char;
							else if(%readValue == 1) %value = %value @ %char;
							%escaped = 0;
						}
						else if(%char $= "\"")
						{
							if(%readVar < 2) %readVar++;
							else %readValue++;
						}
						else if(%char $= "\\")
							%escaped = 1;
						else if(%readVar == 1) %varName = %varName @ %char;
						else if(%readValue == 1) %value = %value @ %char;
					}
					%brick.setVar(%varName, %value);
			}
			continue;
		}
		else if(isObject(%core))
		{
			%uiName = getSubStr(%line, 0, %pos = strPos(%line, "\""));
			if(!isObject(%data = $uiNameTable[%uiName]))
			{ if(!%printedError[%uiName]) { %printedError[%uiName] = 1; error("Unknown brick \""@%uiName@"\"");} continue; }
			%line = getSubStr(%line, %pos + 2, strLen(%line) - (%pos + 2));
			//%brick = createBrick(%group, %data, getWords(%line, 0, 2), getWord(%line, 5), getWord(%line, 3));
			%brick = createBrick(%group, %data, vectorAdd(getWords(%line, 0, 2), %center), getWord(%line, 5), getWord(%line, 3));
			if((%err = getField(%brick, 1)) && %err != 2) { %brick.delete(); continue; }
			%brick.isSteel = 1; %brick.stationCore = %core; %core.brickgroup.add(%brick);
			%brick.setPrint($PrintNameTable[getWord(%line, 6)]); %brick.setColorFX(getWord(%line, 7)); %brick.setShapeFX(getWord(%line, 8));
			%brick.setRaycasting(getword(%line, 9)); %brick.setColliding(getWord(%line, 10)); %brick.setRendering(getWord(%line, 11));
			%brick.getDatablock().onAdd(%brick);
			if((%type = %brick.getDatablock().specialBrickType) !$= "")
			{
				switch$(%type)
				{
					case "Spawnpoint":
						%group.addSpawnBrick(%brick);
					case "RSM_Fabricator":
						%cover = createBrick(%group, brickFabricatorCoverData, %brick.getPosition(), %brick.getColorID(), %brick.getAngleID());
						%cover.mainFabber = %brick; %brick.fabberCover = %cover;
				}
			}
		}
		else if(%line $= "TAB_BLID:")
			%readGroupNames = 1;
		else if(%readGroupNames)
			%nameOf[firstWord(%line)] = restWords(%line);
	}
	%file.close(); %file.delete();
}

function RSM_SaveToFile(%path)
{
	%file = new FileObject(); %file.openForWrite(%path);
	if(!isObject(%cores = StationCores) || (%count = %cores.getCount()) == 0) { %file.writeLine(""); return; }
	for(%i=0;%i<64;%i++) %file.writeLine(getColorIDTable(%i));
	%groups = MainBrickgroup.getCount();
	for(%i=0;%i<%groups;%i++)
	{
		%group = MainBrickgroup.getObject(%i);
		%dontSaveName1337 = 1;
		%dontSaveName888888 = 1;
		%dontSaveName999999 = 1;
		if(%dontSaveName[%group.bl_id]) continue;
		if(!%wroteBLIDTable)
		{
			%file.writeLine("TAB_BLID:");
			%wroteBLIDTable = 1;
		}
		%file.writeLine(%group.bl_id SPC %group.name);
	}
	for(%i=0;%i<%count;%i++)
	{
		%core = StationCores.getObject(%i);
		%center = %core.getPosition();
		%file.writeLine("Station "@%core.stationName);
		%file.writeLine("\t+-CORE "@%core.getGroup().bl_id SPC vectorScale(%center, 60 / $RSM::AsteroidDist) SPC %core.getColorID() SPC %core.getColorFXID()
			SPC %core.getShapeFXID() SPC %core.isRaycasting() SPC %core.isColliding() SPC %core.isRendering());
		%matList = materialList();
		%matCount = getFieldCount(%matList);
		for(%j=0;%j<%matCount;%j++)
		{
			%mat = getField(%matList, %j);
			%file.writeLine("\t+-MAT "@%mat TAB mFloatLength(%core.mat[%mat], 3));
		}
		%bricks = %core.brickgroup.getCount();
		for(%j=-1;%j<%bricks;%j++)
		{
			if(%j == -1) %brick = %core;
			else
			{
				%brick = %core.brickgroup.getObject(%j);
				if(%brick.getDatablock().uiName $= "") continue;
				%file.writeLine("\t"@%brick.getDatablock().uiName@"\" "@vectorSub(%brick.getPosition(), %center)
					SPC %brick.getAngleID()@" 0 "@%brick.getColorID() SPC %brick.getPrintName()
					SPC %brick.getColorFXID() SPC %brick.getShapeFXID() SPC %brick.isRaycasting()
					SPC %brick.isColliding() SPC %brick.isRendering());
			}
			if((%name = %brick.getName()) !$= "") %file.writeLine("\t+-NTOBJECTNAME "@getSubStr(%name, 1, strLen(%name) - 1));
			for(%k=0;%k<%brick.vars;%k++)
			{
				%var = %brick.var[%k];
				%value = %brick.value[%var];
				%var = strReplace(strReplace(%var, "\\", "\\\\"), "\"", "\\\"");
				%value = strReplace(strReplace(%value, "\\", "\\\\"), "\"", "\\\"");
				%file.writeLine("\t+-VALUE \""@%var@"\" = \""@%value@"\";");
			}
			if(isObject(%emit = %brick.emitter)) %file.writeLine("\t+-EMITTER "@%emit.emitter.uiName@"\" "@%brick.emitterDirection);
			if(isObject(%light = %brick.light)) %file.writeLine("\t+-LIGHT "@%light.getDatablock().uiName@"\" 1");
			if(isObject(%item = %brick.item)) %file.writeLine("\t+-ITEM "@%item.getDatablock().uiName@"\" "@%brick.itemPosition SPC %brick.itemDirection SPC %brick.itemRespawnTime);
			if(isObject(%audio = %brick.audioEmitter)) %file.writeLine("\t+-AUDIOEMITTER "@%audio.profile.uiName);
			if(isObject(%vehicle = %brick.vehicle)) %file.writeLine("\t+-VEHICLE "@%vehicle.getDatablock().uiName@"\" "@%brick.recolorVehicle);
			for(%k=0;%k<%brick.numEvents;%k++)
			{
				//Figure out a way to detect datablocks and similar so that they're not destroyed if what's loaded is changed
				%params = ""; for(%l=0;%l<5;%l++) %params = %params TAB %brick.eventOutputParameter[%k, %l + 1];
				%file.writeLine("\t+-EVENT\t"@%brick.eventEnabled[%k] TAB %brick.eventDelay[%k] TAB %brick.eventInput[%k]
					TAB (%brick.eventTarget[%k]==-1?"-1\t"@%brick.eventNT[%k]:%brick.eventTarget[%k]) TAB %brick.eventOutput[%k] @ %params);
			}
		}
		//Dataname" X Y Z AngleID OnGround ColorID Printname ColorFX ShapeFX Ray Col Rend
	}
	%file.close(); %file.delete();
}

//Author for getPrintName: Chrono
function fxDtsBrick::getPrintName(%brick)
{
	if(!%brick.getDatablock().hasPrint) return "";
	%texture = getPrintTexture(%brick.getPrintID());
	%package = getField(strreplace(%texture,"/","\t"),1);
	%category = getField(strReplace(%package,"_","\t"),1);
	%name = fileBase(%texture);
	return %category @ "/" @ %name;
}

function stripLeadingTags(%text) { while(strLen(%text) > 0 && getSubStr(%text, 0, 1) $= "\t") %text = getSubStr(%text, 1, strLen(%text) - 1); return %text; }

function RSM_Autosave()
{
	cancel($RSM_Autosave);
	RSM_SaveToFile("config/server/RSM Autosave "@strReplace(strReplace(getDateTime(), "/", "-"), ":", ".")@".rss");
	$RSM_Autosave = schedule(900000, 0, "RSM_Autosave");
}
if(!isEventPending($RSM_Autosave))
	$RSM_Autosave = schedule(900000, 0, "RSM_Autosave");